' SmartGetSizes:
' Revision: v1.2.2 (vb)
' (c) 2015 steje
'
' This is a script for written for Directory Opus v11.
' See http://www.gpsoft.com.au/redirect.asp?page=scripts for development information.

' =====================================================================================
' =========================   SCRIPT COMMAND: SmartGetSizes   =========================
'
' This script adds a 'SmartGetSizes' RAW command to Directory Opus.
'
' The command is adapted from a solution Jonathan Potter wrote for a user who asked for
' a way to make the normal 'GetSizes' command work normally for any folders selected
' without deselecting them on 'first' use, but would then subsequently calculate sizes
' for all remaining folders in the current tab if used a 'second' time.
'
' That clever code has been modified here both for the purposes of being used by an
' event handling script called OnAfterFolderChange_smartgetsizes, as well as to provide
' an additional RAW command option (GETALL/S) to the functionality of the script function
' Jon had originally devised. The GETALL simply forces size calculation for ALL folders
' upon 'first' use (without first calculating sizes for ~only the selected folders by
' ignoring the size calculation state of any selected folders)...
'
' =====================================================================================

Option Explicit
Dim SCRIPT_DEBUG
SCRIPT_DEBUG = False


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called by Directory Opus to initialize the script
Function OnInit(ByRef initData)

   DOpus.Output("Initializing...")

   ' Provide basic information about the Script
   initData.min_version = "11.16"
   initData.name = "SmartGetSizes"
   initData.desc = "Adds the 'SmartGetSizes' command to Directory Opus."
   initData.copyright = "(c) 2015 steje"
   initData.version = "v1.2.2 (vb)"
   initData.default_enable = True

   '////////////////////////////////////////////////
   ' The following options are user configurable via the Configure button under the scripts listing in Prefs
   '////////////////////////////////////////////////
   ' Set DEBUG flag to True in order to enable logging messages to the Opus Output Window
   initData.config.DEBUG = False
   '////////////////////////////////////////////////
   ' Set DEBUG_CLEAR flag to True in order to clear log messages from the Opus Output Window between script runs
   initData.config.DEBUG_CLEAR = False
   '////////////////////////////////////////////////
   ' Set descriptions for all script options
   initData.config_desc = DOpus.NewMap("DEBUG", "Set this option to True to enable logging to the Opus script log.", _
                                       "DEBUG_CLEAR", "Set this option to True clear messages logged to the Opus script log between script runs.")

   ' Initialize the command that this script adds
   Dim cmd
   Set cmd = initData.AddCommand()
   cmd.name = "SmartGetSizes"
   cmd.icon = "getsizes"
   cmd.method = "OnSmartGetSizes"
   cmd.desc = "Calculates folder sizes for selected folders on first use, then all remaining folders on " &_
              "second use." & vbCrLf & vbCrLf &_
              "Supports the use of a GETALL option to force size calculation of all folders, ignoring folder " &_
              "selection state."
   cmd.label = "SmartGetSizes"
   cmd.template = "GETALL/S,NODESELECT/S"

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Implement the SmartGetSizes command
Function OnSmartGetSizes(ByRef commandData)
   Dim fGetAll, fNoDeselect
   Dim objSrcTabCmd, file

   fGetAll = False
   fNoDeselect = False

   Set objSrcTabCmd = DOpus.NewCommand
   If (objSrcTabCmd.IsSet("$glob:SCRIPT_DEBUG")) Then SCRIPT_DEBUG = True
   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) And (Script.config.DEBUG_CLEAR) Then DOpus.ClearOutput
   'logMsg("OnSmartGetSizes()")

   logMsg("")
   logMsg("----------------------------------------------------------------")
   logMsg("OnSmartGetSizes:")
   logMsg("----------------------------------------------------------------")
   ' Check for the GETALL option
   If (commandData.func.args.got_arg.getall) Then fGetAll = True
   ' Check for the NODESELECT option
   If (commandData.func.args.got_arg.nodeselect) Then fNoDeselect = True

   ' If the GETALL option was NOT used, check to see if any folders are selected that have NOT already had their size calc'd
   If Not (fGetAll) Then
      ' Set the flag to get all folder sizes right away since that's what we want if all selected folders have already had
      ' their sizes calc'd. If any selected folders have NOT had their size calc'd we set the flag to false again and exit...
      fGetAll = True

      ' Loop through selected items to check if any are folders that have not already had sizes calc'd
      For Each file in (commandData.func.command.files)
         If (file.is_dir) And (Not (file.got_size)) Then
            logMsg("At least one folder without its size calculated was selected")
            fGetAll = False
            Exit For
         End If
      Next
   End If

   If (fGetAll) Then commandData.func.command.ClearFiles
   If (fNoDeselect) Then commandData.func.command.deselect = False
   commandData.func.command.RunCommand "GetSizes"

   Set objSrcTabCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Subroutine that uses DEBUG variables to control whether logging is performed or not
Sub logMsg(ByRef message)

   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) Then DOpus.Output(message)

End Sub
